@@ -13,6 +13,7 @@ static IMForwardEventStruct *current_forward_eve;
 
 static char callback_str_buffer[32];
 Window focus_win;
+static int timeout_handle;
 
 static void send_fake_key_eve()
 {
@@ -52,6 +53,12 @@ void clear_output_buffer()
 
 gboolean gb_output = FALSE;
 
+void toggle_gb_output()
+{
+  gb_output = !gb_output;
+}
+
+
 void send_text(char *text)
 {
   if (!text)
@@ -187,6 +194,11 @@ void hide_in_win(ClientState *cs)
   dbg("hide_in_win %d\n", ic->in_method);
 #endif
 
+  if (timeout_handle) {
+    g_source_remove(timeout_handle);
+    timeout_handle = 0;
+  }
+
   switch (cs->in_method) {
     case 3:
       hide_win_pho();
@@ -195,7 +207,7 @@ void hide_in_win(ClientState *cs)
 //      flush_tsin_buffer();
       hide_win0();
       break;
-    case 0:
+    case 10:
       hide_win_int();
       break;
     default:
@@ -203,7 +215,9 @@ void hide_in_win(ClientState *cs)
   }
 
   reset_current_in_win_xy();
+#if 0
   hide_win_status();
+#endif
 }
 
 void show_win_pho();
@@ -227,16 +241,18 @@ void show_in_win(ClientState *cs)
     case 6:
       show_win0();
       break;
-    case 0:
+    case 10:
       show_win_int();
       break;
     default:
       show_win_gtab();
   }
-
+#if 0
   show_win_stautus();
+#endif
 }
 
+
 void move_win_gtab(int x, int y);
 void move_win_int(int x, int y);
 void move_win0(int x, int y);
@@ -272,7 +288,7 @@ void move_in_win(ClientState *cs, int x, int y)
     case 6:
       move_win0(x, y);
       break;
-    case 0:
+    case 10:
       move_win_int(x, y);
       break;
     default:
@@ -372,9 +388,11 @@ void win_pho_disp_half_full();
 void win_tsin_disp_half_full();
 void win_gtab_disp_half_full();
 extern char eng_full_str[], full_char_str[];
+void update_tray_icon();
 
 void disp_im_half_full()
 {
+#if 0
   if (current_CS->im_state == GCIN_STATE_ENG_FULL) {
      set_win_status_half_full(eng_full_str);
   }
@@ -383,6 +401,8 @@ void disp_im_half_full()
      set_win_status_half_full(current_CS->b_half_full_char?full_char_str:"");
   else
      set_win_status_half_full("");
+#endif
+  update_tray_icon();
 
   switch (current_CS->in_method) {
     case 3:
@@ -406,7 +426,7 @@ void toggle_im_enabled(u_int kev_state)
     check_CS();
 
     if (current_CS->in_method < 0 || current_CS->in_method >= gcin_switch_keysN)
-      return;
+      p_err("err found");
 
     static u_int orig_caps_state;
 
@@ -431,8 +451,12 @@ void toggle_im_enabled(u_int kev_state)
       }
 
       hide_in_win(current_CS);
+#if 0
       hide_win_status();
+#endif
       current_CS->im_state = GCIN_STATE_DISABLED;
+
+      update_tray_icon();
     } else {
       current_CS->im_state = GCIN_STATE_CHINESE;
       orig_caps_state = kev_state & LockMask;
@@ -449,9 +473,11 @@ void toggle_im_enabled(u_int kev_state)
       update_in_win_pos();
       show_in_win(current_CS);
 #endif
-
+#if 0
       if (gcin_pop_up_win)
         show_win_stautus();
+#endif
+      update_tray_icon();
     }
 }
 
@@ -469,7 +495,7 @@ void update_active_in_win_geom()
     case 6:
       get_win0_geom();
       break;
-    case 0:
+    case 10:
       break;
     default:
       get_win_gtab_geom();
@@ -527,6 +553,7 @@ void check_CS()
 }
 
 
+void load_tray_icon();
 gboolean init_in_method(int in_no)
 {
   gboolean status = TRUE;
@@ -542,6 +569,7 @@ gboolean init_in_method(int in_no)
       flush_tsin_buffer();
 
     hide_in_win(current_CS);
+
   }
 
   reset_current_in_win_xy();
@@ -557,7 +585,7 @@ gboolean init_in_method(int in_no)
     case 6:
       init_tab_pp(True);
       break;
-    case 0:
+    case 10:
       init_inter_code(True);
       break;
     default:
@@ -566,7 +594,10 @@ gboolean init_in_method(int in_no)
       break;
   }
 
+#if 0
   set_win_status_inmd(inmd[in_no].cname);
+#endif
+  load_tray_icon();
 
   update_in_win_pos();
   return status;
@@ -614,6 +645,14 @@ int feedkey_intcode(KeySym key);
 void tsin_set_eng_ch(int nmod);
 static KeySym last_keysym;
 
+gboolean timeout_raise_window()
+{
+  timeout_handle = 0;
+  show_in_win(current_CS);
+  return FALSE;
+}
+
+
 // return TRUE if the key press is processed
 gboolean ProcessKeyPress(KeySym keysym, u_int kev_state)
 {
@@ -677,6 +716,12 @@ gboolean ProcessKeyPress(KeySym keysym, u_int kev_state)
       return TRUE;
     }
 
+    if (keysym == ',') {
+      create_win_sym();
+
+      return TRUE;
+    }
+
     int kidx = gcin_switch_keys_lookup(keysym);
     if (kidx < 0)
       return FALSE;
@@ -701,13 +746,16 @@ gboolean ProcessKeyPress(KeySym keysym, u_int kev_state)
      return TRUE;
   }
 
+  if (current_CS->b_raise_window && !timeout_handle) {
+    timeout_handle = g_timeout_add(200, timeout_raise_window, NULL);
+  }
 
   switch(current_CS->in_method) {
     case 3:
       return feedkey_pho(keysym, kev_state);
     case 6:
       return feedkey_pp(keysym, kev_state);
-    case 0:
+    case 10:
       return feedkey_intcode(keysym);
     default:
       return feedkey_gtab(keysym, kev_state);
@@ -797,12 +845,14 @@ int gcin_FocusIn(ClientState *cs)
       show_in_win(cs);
       move_IC_in_win(cs);
 #endif
+#if 0
       set_win_status_inmd(inmd[cs->in_method].cname);
+#endif
     } else
       hide_in_win(cs);
   }
 
-
+  load_tray_icon();
 #if 0
   dbg_time("gcin_FocusIn %x %x\n",cs, current_CS);
 #endif
