@@ -23,6 +23,7 @@
 #define GCIN_FLAGS_IM_ENABLED "gcin-flags-im-enabled"
 #define GCIN_REMOTE_CLIENT "gcin-remote-client"
 #define GCIN_SHIFT_SPACE_ENG_FULL "gcin-shift-space-eng-full"
+#define GCIN_STATUS_TRAY "gcin-status-tray"
 
 
 #define GTAB_DUP_SELECT_BELL "gtab-dup-select-bell"
@@ -38,6 +39,7 @@
 #define GTAB_SHIFT_PHRASE_KEY "gtab-shift-phrase-key"
 #define GTAB_HIDE_ROW2 "gtab-hide-row2"
 #define GTAB_IN_ROW1 "gtab-in-row1"
+#define GTAB_CAPSLOCK_IN_ENG "gtab-capslock-in-eng"
 
 #define PHO_SIMPLE_WIN "pho-simple-window" // common to both pho and tsin
 #define TSIN_PHRASE_PRE_SELECT "tsin-phrase-pre-select"
@@ -48,6 +50,8 @@
 #define TSIN_PHRASE_LINE_COLOR "tsin-phrase-line-color"
 #define TSIN_CURSOR_COLOR "tsin-cursor-color"
 #define TSIN_TONE_CHAR_INPUT "tsin-tone-char-input"
+#define TSIN_TAB_PHRASE_END "tsin-tab-phrase-end"
+
 #define PHO_HIDE_ROW2 "pho-hide-row2"
 #define PHO_IN_ROW1 "pho-in-row1"
 
@@ -62,7 +66,8 @@ extern int gcin_font_size, gcin_font_size_tsin_presel, gcin_font_size_symbol,
            gcin_font_size_gtab_in, gcin_inner_frame,
            gcin_flags_im_enabled, gcin_remote_client, gtab_simple_win,
            gtab_disp_key_codes, gtab_disp_im_name, gcin_shift_space_eng_full,
-           gtab_invalid_key_in, gtab_hide_row2, gtab_in_row1, pho_hide_row2,
+           gtab_invalid_key_in, gtab_hide_row2, gtab_in_row1, gtab_capslock_in_eng,
+           pho_hide_row2,
            pho_in_row1;
 
 extern int default_input_method;
@@ -78,9 +83,11 @@ extern int gtab_shift_phrase_key;
 extern int tsin_ec_toggle_key;
 extern int pho_simple_win, tsin_buffer_size;
 extern int gcin_input_style, gcin_root_x, gcin_root_y, gcin_pop_up_win, gcin_pop_up_win_abs_corner;
+extern int gcin_status_tray;
 
 extern int tsin_phrase_pre_select;
 extern int tsin_chinese_english_toggle_key;
+extern int tsin_tab_phrase_end;
 
 extern int phonetic_char_dynamic_sequence;
 extern int phonetic_huge_tab;
@@ -93,3 +100,5 @@ void get_gcin_conf_str(char *name, char rstr[], char *default_str);
 void save_gcin_conf_str(char *name, char *str);
 void save_gcin_conf_int(char *name, int val);
 void load_setttings();
+
+#define TRAY_ENABLED 1
