@@ -112,6 +112,7 @@ void utf8cpy(char *t, char *s);
 int u8cpy(char *t, char *s);
 int utf8_tlen(char *s, int N);
 void utf8_putchar(char *s);
+void utf8_putcharn(char *s, int n);
 gboolean utf8_eq(char *a, char *b);
 gboolean utf8_str_eq(char *a, char *b, int len);
 void utf8cpyN(char *t, char *s, int N);
@@ -138,3 +139,5 @@ extern char gcin_switch_keys[];
 #if GTK_MAJOR_VERSION >=2 && GTK_MINOR_VERSION >= 4
 #define GTK_24 1
 #endif
+
+typedef int usecount_t;
