@@ -266,15 +266,20 @@ void load_gtab_list()
   dbg("load_gtab_list %s\n", ttt);
 
   while (!feof(fp)) {
+    char line[256];
     char name[32];
     char key[32];
     char file[32];
+    char icon[128];
 
     name[0]=0;
     key[0]=0;
     file[0]=0;
+    icon[0]=0;
 
-    fscanf(fp, "%s %s %s", name, key, file);
+    fgets(line, sizeof(line), fp);
+
+    sscanf(line, "%s %s %s %s", name, key, file, icon);
     if (strlen(name) < 1)
       break;
     if (name[0]=='#')
@@ -289,6 +294,10 @@ void load_gtab_list()
 
     free(inmd[keyidx].cname);
     inmd[keyidx].cname = strdup(name);
+
+    free(inmd[keyidx].icon);
+    if (strlen(icon))
+      inmd[keyidx].icon = strdup(icon);
   }
 
   fclose(fp);
@@ -960,18 +969,6 @@ gboolean feedkey_gtab(KeySym key, int kbstate)
 
   if (same_pho_query_state == SAME_PHO_QUERY_pho_select)
     return feedkey_pho(key, 0);
-#if 0
-  if (current_CS->b_half_full_char) {
-     char *s = half_char_to_full_char(key);
-     if (!s)
-       return 0;
-     char tt[CH_SZ+1];
-
-     memcpy(tt, s, CH_SZ); tt[CH_SZ]=0;
-     send_text(tt);
-     return 1;
-  }
-#endif
 
 
   if (key < 127 && cur_inmd->keymap[key]) {
@@ -981,8 +978,12 @@ gboolean feedkey_gtab(KeySym key, int kbstate)
      char ucase = toupper(key);
      if (cur_inmd->keymap[lcase] != cur_inmd->keymap[ucase])
        goto next;
+
+     if (gtab_capslock_in_eng && (kbstate&LockMask))
+       return 0;
   }
 
+
 shift_proc:
   if ((kbstate & ShiftMask) && key!='*' && (key!='?' || gtab_shift_phrase_key && !ci)) {
     if (gtab_shift_phrase_key)
@@ -1156,6 +1157,7 @@ shift_proc:
       }
     default:
 next:
+
       if (invalid_spc) {
         ClrIn();
       }
