@@ -1,5 +1,5 @@
 /*
-	Copyright (C) 2004	Edward Der-Hua Liu, Hsin-Chu, Taiwan
+	Copyright (C) 2004	Edward Liu, Taiwan
 */
 #include <string.h>
 
@@ -24,23 +24,8 @@ extern PHOKBM phkbm;
 extern char tsidxfname[64];
 extern int hashidx[TSIN_HASH_N];
 
-typedef struct {
-  phokey_t pho;
-  char ch[CH_SZ];
-  char och[CH_SZ];
-  char ph1ch[CH_SZ]; // char selected by 1st pho
-  u_char flag;
-  char psta; // phrase start index
-} CHPHO;
-
-enum {
-  FLAG_CHPHO_FIXED=1,    // user selected the char, so it should not be changed
-  FLAG_CHPHO_PHRASE_HEAD=2,
-  FLAG_CHPHO_PHRASE_VOID=4
-};
-
-static CHPHO chpho[MAX_PH_BF_EXT];
-static int c_idx, c_len, ph_sta=-1, ph_sta_last=-1;
+CHPHO chpho[MAX_PH_BF_EXT];
+int c_idx, c_len, ph_sta=-1, ph_sta_last=-1;
 static int sel_pho;
 static gboolean eng_ph=TRUE;  // english(FALSE) <-> pho(juyin, TRUE)
 static int save_frm, save_to;
@@ -54,7 +39,7 @@ typedef struct {
   int phidx;
   char str[MAX_PHRASE_LEN*CH_SZ+1];
   int len;
-  char usecount;
+  usecount_t usecount;
 } PRE_SEL;
 
 static PRE_SEL pre_sel[10];
@@ -112,8 +97,10 @@ void drawcursor()
 
   if (c_idx == c_len) {
     if (tsin_half_full || eng_ph) {
+#if 0
       disp_char(c_idx,"  ");
       set_cursor_tsin(c_idx);
+#endif
     } else {
       disp_char(c_idx, " ");
       set_cursor_tsin(c_idx);
@@ -151,11 +138,13 @@ static void putbuf(int len)
   u_char tt[CH_SZ * (MAX_PH_BF_EXT+1) + 1];
   int i,idx;
 
+//  dbg("putbuf:%d\n", len);
 
 #if 1
   // update phrase reference count
   if (len >= 2) {
     for(i=0; i < len; i++) {
+//      dbg("flag %d %x\n", i, chpho[i].flag);
       if (!BITON(chpho[i].flag, FLAG_CHPHO_PHRASE_HEAD)) {
         continue;
       }
@@ -177,7 +166,7 @@ static void putbuf(int len)
       puts("");
 #endif
 
-      if (phrlen < 2)
+      if (phrlen < 1)
         continue;
 
       phokey_t pho[MAX_PHRASE_LEN];
@@ -224,11 +213,13 @@ static void prbuf()
     disp_char_chbuf(i);
   }
 
+#if 0
   if (c_len < MAX_PH_BF_EXT)
     hide_char(c_len);
 
   if (c_len+1 < MAX_PH_BF_EXT)
     hide_char(c_len+1);
+#endif
 
   drawcursor();
 }
@@ -333,8 +324,6 @@ void show_stat()
 
 void load_tsin_db();
 
-void load_tsin_entry(int idx, u_char *len, char *usecount, phokey_t *pho,
-                    u_char *ch);
 
 #if 0
 void nputs(u_char *s, u_char len)
@@ -351,7 +340,7 @@ static void dump_tsidx(int i)
 {
   phokey_t pho[MAX_PHRASE_LEN];
   u_char ch[MAX_PHRASE_LEN*CH_SZ];
-  char usecount;
+  usecount_t usecount;
   u_char len;
 
   load_tsin_entry(i, &len, &usecount, pho, ch);
@@ -416,14 +405,11 @@ gboolean save_phrase_to_db2(CHPHO *chph, int len);
 
 static void save_phrase()
 {
-  int tt, i;
+  int i;
   u_char len;
 
-  if (save_frm==save_to)
-    return;
-
   if (save_frm > save_to) {
-    tt=save_frm;
+    int tt=save_frm;
     save_frm=save_to;
     save_to=tt;
   }
@@ -432,7 +418,8 @@ static void save_phrase()
     save_to--;
 
   len= save_to- save_frm+ 1;
-  if (len < 2 || len > MAX_PHRASE_LEN)
+
+  if (len <= 0 || len > MAX_PHRASE_LEN)
     return;
 
   for(i=save_frm;i<=save_to;i++) {
@@ -561,7 +548,7 @@ static void get_sel_phrase()
   phrase_count = 0;
 
   while (sti < edi && phrase_count < MAX_PHRASE_SEL_N) {
-    char usecount=0;
+    usecount_t usecount=0;
     load_tsin_entry(sti, &len, &usecount, stk, stch);
 
     if (len > mlen) {
@@ -569,7 +556,6 @@ static void get_sel_phrase()
       continue;
     }
 
-
     if (chpho_eq_pho(c_idx, stk, len)) {
       sellen[phrase_count]=len;
       selidx[phrase_count]=sti;
@@ -580,8 +566,6 @@ static void get_sel_phrase()
   }
 }
 
-
-
 static void get_sel_pho()
 {
   phokey_t key;
@@ -646,7 +630,7 @@ static void disp_current_sel_page()
   if (current_page > 0)
     disp_arrow_up();
 
-  disp_selections(c_idx);
+  disp_selections(c_idx==c_len?c_idx-1:c_idx);
 }
 
 static int fetch_user_selection(int val, char **seltext)
@@ -672,7 +656,7 @@ static int fetch_user_selection(int val, char **seltext)
 
 int phokey_t_seq(phokey_t *a, phokey_t *b, int len);
 
-static void extract_pho(int chpho_idx, int plen, phokey_t *pho)
+void extract_pho(int chpho_idx, int plen, phokey_t *pho)
 {
   int i;
 
@@ -682,7 +666,6 @@ static void extract_pho(int chpho_idx, int plen, phokey_t *pho)
 }
 
 
-gboolean tsin_seek(phokey_t *pho, int plen, int *r_sti, int *r_edi);
 void mask_key_typ_pho(phokey_t *key);
 
 static int qcmp_pre_sel_usecount(const void *aa, const void *bb)
@@ -693,6 +676,27 @@ static int qcmp_pre_sel_usecount(const void *aa, const void *bb)
   return b->usecount - a->usecount;
 }
 
+gboolean check_fixed_mismatch(int chpho_idx, char *mtch, int plen)
+{
+  int j;
+  char *p = mtch;
+
+  for(j=0; j < plen; j++) {
+    int u8sz = utf8_sz(p);
+    if (!(chpho[chpho_idx+j].flag & FLAG_CHPHO_FIXED))
+      continue;
+
+    if (memcmp(chpho[chpho_idx+j].ch, p, u8sz))
+      break;
+
+    p+= u8sz;
+  }
+
+  if (j < plen)
+    return TRUE;
+
+  return FALSE;
+}
 
 static u_char scanphr(int chpho_idx, int plen, gboolean pho_incr)
 {
@@ -730,12 +734,11 @@ static u_char scanphr(int chpho_idx, int plen, gboolean pho_incr)
   while (sti < edi && pre_selN < 10) {
     phokey_t mtk[MAX_PHRASE_LEN];
     u_char mtch[MAX_PHRASE_LEN*CH_SZ+1];
-    u_char match_len;
-    char usecount;
+    char match_len;
+    usecount_t usecount;
 
     load_tsin_entry(sti, &match_len, &usecount, mtk, mtch);
 
-
     sti++;
     if (plen > match_len || (pho_incr && plen==match_len)) {
       continue;
@@ -776,21 +779,7 @@ static u_char scanphr(int chpho_idx, int plen, gboolean pho_incr)
 #endif
 
 #if 1
-    int j;
-    char *p = mtch;
-
-    for(j=0; j < plen; j++) {
-      int u8sz = utf8_sz(p);
-      if (!(chpho[chpho_idx+j].flag & FLAG_CHPHO_FIXED))
-        continue;
-
-      if (memcmp(chpho[chpho_idx+j].ch, p, u8sz))
-        break;
-
-      p+= u8sz;
-    }
-
-    if (j < plen)
+    if (check_fixed_mismatch(chpho_idx, mtch, plen))
       continue;
 #endif
 
@@ -819,7 +808,7 @@ static void disp_pre_sel_page()
     return;
   }
 
-  if (pre_selN==0 || (pre_selN==1 && pre_sel[0].len==2))
+  if (pre_selN==0 || (pre_selN==1 && pre_sel[0].len<=2))
     return;
 
   clear_sele();
@@ -829,7 +818,9 @@ static void disp_pre_sel_page()
 
     set_sele_text(i, pre_sel[i].str, tlen);
   }
-
+#if 0
+  dbg("ph_sta:%d\n", ph_sta);
+#endif
   disp_selections(ph_sta);
 }
 
@@ -975,23 +966,6 @@ gboolean add_to_tsin_buf(char *str, phokey_t *pho, int len)
     if (c_idx < 0 || c_len + len >= MAX_PH_BF_EXT)
       return 0;
 
-
-    char *pp = str;
-#if 0
-    for(i=0; i < len; i++) {
-      if (pho[i])
-        continue;
-
-      phokey_t tpho[32];
-      tpho[0]=0;
-      int u8len = utf8_sz(pp);
-
-      utf8_pho_keys(pp, tpho);
-      pho[i] = tpho[0];
-      pp += u8len;
-    }
-#endif
-
     if (c_idx < c_len) {
       for(i=c_len-1; i >= c_idx; i--) {
         chpho[i+len] = chpho[i];
@@ -1014,10 +988,10 @@ gboolean add_to_tsin_buf(char *str, phokey_t *pho, int len)
       chpho[c_idx+i].psta= c_idx;
     }
 #endif
-
-    if (len > 1)
+#if 0
+    if (len > 0)
       chpho[c_idx].flag |= FLAG_CHPHO_PHRASE_HEAD;
-
+#endif
     drawcursor();
     disp_ph_sta();
     hide_pre_sel();
@@ -1029,11 +1003,12 @@ gboolean add_to_tsin_buf(char *str, phokey_t *pho, int len)
     return TRUE;
 }
 
+#if 1
 static void set_phrase_link(int idx, int len)
 {
     int j;
 
-    if (len < 2)
+    if (len < 1)
       return;
 
     for(j=1;j < len; j++) {
@@ -1042,6 +1017,14 @@ static void set_phrase_link(int idx, int len)
 
     chpho[idx].flag |= FLAG_CHPHO_PHRASE_HEAD;
 }
+#endif
+
+static void set_fixed(int idx, int len)
+{
+  int i;
+  for(i=idx; i < idx+len; i++)
+    chpho[i].flag |= FLAG_CHPHO_FIXED;
+}
 
 // should be used only if it is a real phrase
 gboolean add_to_tsin_buf_phsta(char *str, phokey_t *pho, int len)
@@ -1060,15 +1043,18 @@ gboolean add_to_tsin_buf_phsta(char *str, phokey_t *pho, int len)
     ch_pho_cpy(&chpho[idx], str, pho, len);
     set_chpho_ch2(&chpho[idx], str, len);
 
+
+    set_fixed(idx, len);
+
     c_len=c_idx=idx + len;
 
     clrin_pho_tsin();
     disp_in_area_pho_tsin();
 
     prbuf();
-
+#if 1
     set_phrase_link(idx, len);
-
+#endif
     drawcursor();
     disp_ph_sta();
     hide_pre_sel();
@@ -1216,7 +1202,7 @@ int feedkey_pp(KeySym xkey, int kbstate)
   key_press_time = 0;
 
    if (kbstate & (Mod1Mask|Mod1Mask)) {
-       return 0;
+     return 0;
    }
 
 
@@ -1291,9 +1277,14 @@ int feedkey_pp(KeySym xkey, int kbstate)
           return 1;
         }
 
-        if (c_len) {
-          flush_tsin_buffer();
+        if (tsin_tab_phrase_end) {
+          ph_sta = -1;
           return 1;
+        } else {
+          if (c_len) {
+            flush_tsin_buffer();
+            return 1;
+          }
         }
         return 0;
      case XK_Shift_L:
@@ -1325,6 +1316,7 @@ int feedkey_pp(KeySym xkey, int kbstate)
         }
 
         c_len--;
+        hide_char(c_len);
         init_chpho_i(c_len);
 
         prbuf();
@@ -1387,6 +1379,7 @@ int feedkey_pp(KeySym xkey, int kbstate)
         }
 
         c_len--;
+        hide_char(c_len);
         init_chpho_i(c_len);
         prbuf();
         compact_win0_x();
@@ -1503,10 +1496,13 @@ other_keys:
            clear_tsin_buffer();
            return 1;
          } else
+         if (xkey>=XK_1 && xkey<=XK_3) {
+           return 1;
+         } else {
            return 0;
+         }
        }
 
-
        char *pp;
 
        char xkey_lcase = xkey;
@@ -1527,13 +1523,17 @@ other_keys:
              set_chpho_ch2(&chpho[c_idx], sel_text, len);
 
            chpho[c_idx].flag &= ~FLAG_CHPHO_PHRASE_VOID;
+#if 0
            set_phrase_link(c_idx, len);
+#endif
+           set_fixed(c_idx, len);
+
            raise_phr(c);
            if (c_idx + len == c_len) {
              ph_sta = -1;
 
              for(i=c_idx; i < c_len; i++) {
-               draw_underline(i);
+                draw_underline(i);
              }
            }
          } else
@@ -1625,9 +1625,8 @@ asc_char:
           }
         }
 
-
         if (xkey > 127)
-          return 1;
+          return 0;
 
         u_char tt=xkey;
         shift_ins();
@@ -1657,8 +1656,9 @@ asc_char:
         drawcursor();
         return 1;
    } else { /* pho */
-     if (xkey > 127)
+     if (xkey > 127) {
        return 0;
+     }
 
      // for hsu & et26
      if (strchr(hsu_punc, xkey) && !phkbm.phokbm[xkey][0].num)
@@ -1741,12 +1741,18 @@ llll2:
 
    put_b5_char(ch_pho[start_idx].ch, key);
 
+   TSIN_PARSE parse[MAX_PH_BF_EXT+1];
+   bzero(parse, sizeof(parse));
+   tsin_parse(parse);
+   prbuf();
+
    disp_ph_sta();
    clrin_pho_tsin();
    clr_in_area_pho_tsin();
    drawcursor();
    hide_pre_sel();
 
+
    if (ph_sta < 0) {
 restart:
      if ((match_len=scanphr(c_idx-1,1, FALSE)))
@@ -1853,8 +1859,9 @@ restart:
        }
 
        full_match = TRUE;
+#if 0
        chpho[ph_sta].flag |= FLAG_CHPHO_PHRASE_HEAD;
-
+#endif
        if (mdist==max_match_phrase_len) { // tstr: 選擇視窗
 //         dbg("full match .......... %d\n", ph_sta);
          ph_sta_last = ph_sta;
